% Program using the formulas of Alemdar & Atalar paper to calculate:
%  * The necessary input levels for desired power output backoff of a Doherty Amplifier 
%  * Center Frequency Performance such as Efficiency, Pout, Currents
% The result is a file BOn.mat that will be used by DohertyFreqResp_m2.m calculating:
%  * Frequency Response Performance
% Please make sure 6 mat files are on the same directory:
%  * DohertyIn?t_m2.m
%  * DohertyFreqResp_m2.m
%  * DrainCurrent1.m
%  * DrainCurrent1m.m
%  * DrainCurrent3.m
%  * DrainCurrent3m.m
clear;close all;

% COMBINING NETWORK SELECTION----------------------------------------------
ConvDo=0; 
% 1 if conventional Doherty amplifier (Conv)
% 0 if two section TL (type I in paper)
% 8 if C-L-C inverter (type II in paper)
% 5 if C-TL-C inverter (type III in paper)
% 6 if C-TL(longer)-C inverter (type III in paper)

% OUTPUT MATCHING NETWORK SELECTION----------------------------------------
OMNNo=2; 
% 1 if one section lambda/4 line (a) in paper
% 2 if two section lambda/4 lines (b) in paper
% 3 if one section CA-TL-CB (e) in paper
% 4 if two section CA-TL-CB (f) in paper
% 6 if one section C-L-C (c) in paper
% 7 if two section C-L-C (d) in paper

fN=1; % Center frequency in GHz 
zeta=3; % First scaling parameter
kappa=1; % Second scaling parameter



% -------------------------------------------------
% TRANSISTOR PARAMETERS

No=15; % order of knee function approximation (odd)
for n=1:(No+1)/2
     nchko(n)=nchoosek(No,(No+1)/2-n);
end 

Ne=14; % order of knee function approximation (even)
for n=0:Ne/2
    nchke(n+1)=nchoosek(Ne,Ne/2-n);
end

VT=-3;        % threshold voltage of RF transistor
VDD=28;       % VDD voltage of RF transistor
Gm=0.6;       % transconductance of RF transistor
Ropt=26.68;   % optimal drain impedance of main amp
Roptp=25.75;  % optimal drain resistance of peaking amp
Ipmax=4.427;  %4.427
VBm=VT+0.001; % bias voltage of carrier amplifier (note: a small shift value is needed for Class B)
VBp=-5.6;     % bias voltage of peaking amplifier (Class-C)
C=1.84e-12;   % drain capacitance 1.84e-12;
Ls=0.26e-9;   % serial package inductance  0.26e-9;
Lsh=14e-9;    % shunt inductance for tuning; 
Ropt=26.6;    % to be used with types II and III in paper
Roptp=25.75;  % to be used with types II and III in paper
% use Ropt given in Table in paper for type I
Ropt=26;
Roptp=Ropt;   %For simplicity

w0=2*pi*fN*1e9;
Q=Ropt/(w0*Lsh);
ZOUT=50; % Load impedance at the output of amp

alpha=1/sqrt(5); % input divider multiplier for main amplifier
beta=sqrt(4)/sqrt(5); % input divider multiplier for peaking amplifier

% END OF TRANSISTOR PARAMETERS
% ------------------------------------------------

%------------------CALCULTIONS BEGIN---------------------------------------

nw=300; % number of input steps of input excitation for interpolation
BOmin=-15; % Minimum output level in dBs
BOmax=0; % Maximum output level in dBs
NoOfSteps=2; % number of  backoff steps
BackOffSmall=0; % defines the largest power (smallest backoff in dB)
StepSize=6; % backoff step size in dB

w=w0; % frequency of evaluation
Zh=100; % maximum TL impedance

% Variables Defined
L=1000;
L1=0;L2=0;L3=0;
C1=0;C2=0;C3=0;C4=0;C_offset=0;L_offset=0;
CA=0;CB=0;CC=0;CD=0;LA=0;LB=0;
x=0;x1=0;y1=0;y2=0;
y=0.25; % by default lambda/4
ZSh=100000;
ySh=0.25;

Z5=ZOUT;
Z6=ZOUT;

%---COMBINING NETWORK CALCULATIONS BEGIN-----------------------------------

switch ConvDo
    case 1 % conventional Doherty amplifier (Conv)
        ConvDoh=1;
        Zcomb=zeta/2*Ropt;
        kappa=zeta*Ropt/Roptp;
        Z2=sqrt(zeta*kappa)*Ropt;
        Z3=0;
        Z4=0;
        
    case 0 % if two section TL (type I in paper)
        ConvDoh=0;
        Zcomb=zeta/2*Ropt;
        Z2=sqrt(zeta*kappa)*Ropt;
        Z3=kappa^0.75*zeta^0.25*Roptp^0.75*Ropt^0.25;
        Z4=kappa^0.25*zeta^0.75*Roptp^0.25*Ropt^0.75;
        
    case 5 % if C-TL-C inverter (type III in paper)
        ConvDoh=0;
        Zcomb=zeta/2*Ropt;
        r=kappa/zeta;
        Z2=1.4*zeta*Ropt;
        C1=(-53*r^3+207*r^2-324*r+240)/(100*zeta*Ropt*w0);
        p=(sqrt(r)-w0*Ls/(zeta*Ropt))^2;
        x=(18.0*p^3-52.8*p^2+132*p+28.8)/(1000);
        if(C1>C)
            C2=C1-C; % extra cap needed in parallel with C
            L1=100; % no shunt inductance needed
        else
            C2=0; % no extra cap needed\
            L1=1/(w0^2*C-w0^2*C1); % L1 is the shunt inductance to partially cancel CC
        end
        Z4=zeta*Ropt;
        r=kappa*Roptp/Z4;
        Z3=Z2;
        C3=(-53*r^3+207*r^2-324*r+240)/(100*zeta*Ropt*w0);
        p=(sqrt(r)-w0*Ls/(zeta*Ropt))^2;
        x1=(18.0*p^3-52.8*p^2+132*p+28.8)/(1000);
        if(C3>C)
            C4=C3-C; % extra cap in parallel
            L3=100000;
        else
            C4=0;% no extra cap
            L3=1/(w0^2*C-w0^2*C3);
        end
        Lsh=1;
        
    case 6 % if C-TL(longer)-C inverter (type III in paper)
        ConvDoh=0;
        Zcomb=zeta/2*Ropt;
        r=kappa/zeta;
        x=(300.95*r^3-497.42*r^2+427.76*r-1.718)/(1000);
        Z2=zeta*Ropt;
        C1=(-337.13*r^3+725.94*r^2-674.74*r+297.9)/(100*zeta*Ropt*w0);
        if(C1>C)
            C2=C1-C; % extra cap needed in parallel with C
            L1=100000; % no shunt inductance needed
        else
            C2=0; % no extra cap needed\
            L1=1/(w0^2*C-w0^2*C1); % L1 is the shunt inductance to partially cancel CC
        end
        Z4=zeta*Ropt;
        r=kappa*Roptp/Z4;
        x1=(300.95*r^3-497.42*r^2+427.76*r-1.718)/(1000);
        Z3=Z2;
        C3=(-337.13*r^3+725.94*r^2-674.74*r+297.9)/(100*zeta*Ropt*w0);
        if(C3>C)
            C4=C3-C; % extra cap in parallel
            L3=100000;
        else
            C4=0;% no extra cap
            L3=1/(w0^2*C-w0^2*C3);
        end
        Lsh=1;
        
    case 8 %  if C-L-C inverter (type II in paper)
        ConvDoh=0;
        % zeta=3;
        Zcomb=zeta/2*Ropt;
        Z2=0;
        L=sqrt(zeta*kappa)*Ropt/w0;
        C1=1/(w0^2*L);
        L=L-Ls; % since Ls is already there
        if(C1>C)
            C2=C1-C; % extra cap needed in parallel with C
            L1=100000; % no shunt inductance needed
        else
            C2=0; % no extra cap needed\
            L1=1/(w0^2*C-w0^2*C1); % L1 is the shunt inductance to partially cancel CC
        end
        Z3=0;
        Z4=zeta*Ropt;
        L2=sqrt(zeta*kappa*Ropt*Roptp)/w0;
        C3=1/(w0^2*L2);
        L2=L2-Ls;
        if(C3>C)
            C4=C3-C; % extra cap in parallel
            L3=100000;
        else
            C4=0;% no extra cap
            L3=1/(w0^2*C-w0^2*C3);
        end
        %Offset line replacement CLC inverter
        L_offset = Roptp*zeta/w0;
        C_offset = 1/(w0*Roptp*zeta);
        Lsh=1; % to eliminate
end

%---COMBINING NETWORK CALCULATIONS END------------------------------------- 
        
%---OUTPUT COMBINING NETWORK CALCULATIONS BEGIN----------------------------
switch OMNNo
    case 1
        Z5=sqrt(Zcomb*ZOUT);
        Z6=ZOUT;
        y1=0.25;
        y2=0;
        
    case 2
        Z5=Zcomb^0.75*ZOUT^0.25;
        Z6=Zcomb^0.25*ZOUT^0.75;
        y1=0.25;
        y2=0.25;
        
    case 3
        ZLBar=Zcomb/ZOUT; % normalized combining resistance
        y1=(9.56*ZLBar^3-43.2*ZLBar^2+83.6*ZLBar+34.7)/(1000);
        %y1=Z5/(2*pi*Zh);
        CA=(1.7*ZLBar^3-7.08*ZLBar^2+10.9*ZLBar-1.09)/(w0*1000);
        CD=0;
        CB=(-10.7*ZLBar^3+44.4*ZLBar^2-74.4*ZLBar+45)/(w0*1000);
        CC=0;
        y2=0;
        Z5=70;
        Z6=ZOUT;
        
    case 4
        Z5=Zcomb^0.75*ZOUT^0.25;
        Zmid=Z5^2/Zcomb;
        ZLBar=Zmid/ZOUT;
        Z6=70;
        y2=(9.56*ZLBar^3-43.2*ZLBar^2+83.6*ZLBar+34.7)/(1000);
        CC=(1.7*ZLBar^3-7.08*ZLBar^2+10.9*ZLBar-1.09)/(w0*1000);
        CD=(-10.7*ZLBar^3+44.4*ZLBar^2-74.4*ZLBar+45)/(w0*1000);
        y1=y2;
        Z5=Zmid/ZOUT*70;
        CA=CC*50/Zmid;
        CB=CD*50/Zmid;
        
    case 6
        LA=sqrt(ZOUT*Zcomb)/w0;
        CA=1/(LA*w0^2);
        CB=CA;
        
    case 7
        LA=(ZOUT^0.25 * Zcomb^0.75)/w0;
        LB=(ZOUT^0.75 * Zcomb^0.25)/w0;
        CA=1/(LA*w0^2);
        CB=CA;
        CC=1/(LB*w0^2);
        CD=CC;
end
%---OUTPUT COMBINING NETWORK CALCULATIONS END------------------------------


% initialize matrices to reserve space
POUT=zeros(1,nw); 
GAIN=POUT;VDSm=POUT;VDSp=POUT;IDSm=POUT;IDSp=POUT;I0m=POUT;I0p=POUT;BO=POUT;

myfun=@sA1;
options = optimoptions('fsolve','Display','none','OptimalityTolerance',1e-7);


switch ConvDo
    case 1  %if conventional Doherty amplifier (Conv)
            Ap=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1];
            Am=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[cos(2*y*pi*w/w0) 1i*Z2*sin(2*y*pi*w/w0);1i/Z2*sin(2*y*pi*w/w0) cos(2*y*pi*w/w0)]; 
    case 0  %if two section TL (type I in paper)
            Am=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[cos(pi/2*w/w0) 1i*Z2*sin(pi/2*w/w0);1i/Z2*sin(pi/2*w/w0) cos(pi/2*w/w0)];  
            Ap=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[cos(pi/2*w/w0) 1i*Z3*sin(pi/2*w/w0);1i/Z3*sin(pi/2*w/w0) cos(pi/2*w/w0)]*[cos(2*y*pi*w/w0) 1i*Z4*sin(2*y*pi*w/w0);1i/Z4*sin(2*y*pi*w/w0) cos(2*y*pi*w/w0)];
     case 8 %if C-L-C inverter (type II in paper)
            Ap=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[1 0; 1i*w*C4 1]*[1 0; -1i/(w*L3) 1]*[1 1i*w*L2; 0 1]*[1 0; 1i*w*C3 1]*[1 0; 1i*w*C_offset 1]*[1 1i*w*L_offset; 0 1]*[1 0; 1i*w*C_offset 1];
            Am=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[1 0; 1i*w*C2 1]*[1 0; -1i/(w*L1) 1]*[1 1i*w*L; 0 1]*[1 0; 1i*w*C1 1];
    case {5,6} %if C-TL-C inverter (type III in paper)
            Ap=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[1 0; 1i*w*C4 1]*[1 0; -1i/(w*L3) 1]*[cos(x1*2*pi*w/w0) 1i*Z3*sin(x1*2*pi*w/w0); 1i/Z3*sin(x1*2*pi*w/w0) cos(x1*2*pi*w/w0)]*[1 0; 1i*w*C3 1]*[cos(2*y*pi*w/w0) 1i*Z4*sin(2*y*pi*w/w0);1i/Z4*sin(2*y*pi*w/w0) cos(2*y*pi*w/w0)];
            Am=[1 0; 1i*w*C 1]*[1 1i*w*Ls; 0 1]*[1 0; 1/(1i*w*Lsh) 1]*[1 0; 1i*w*C2 1]*[1 0; -1i/(w*L1) 1]*[cos(x*2*pi*w/w0) 1i*Z2*sin(x*2*pi*w/w0); 1i/Z2*sin(x*2*pi*w/w0) cos(x*2*pi*w/w0)]*[1 0; 1i*w*C1 1];
    
end           
Zout=ZOUT; % corresponding normalized load impedance of Doherty


    AOMN=...
        ...%shunt component 1
        [1 0; 1i*w*CA 1]*...
        ...%series component 1
        [cos(y1*2*pi*w/w0) 1i*Z5*sin(y1*2*pi*w/w0); 1i/Z5*sin(y1*2*pi*w/w0) cos(y1*2*pi*w/w0)]*...
        [1 1i*w*LA; 0 1]*...
        ...%shunt component 2
        [1 0; 1i*w*(CB+CC) 1]*...
        ...%series component 2
        [cos(y2*2*pi*w/w0) 1i*Z6*sin(y2*2*pi*w/w0); 1i/Z6*sin(y2*2*pi*w/w0) cos(y2*2*pi*w/w0)]*...
        [1 1i*w*LB; 0 1]*...
        ...%shunt component 3    
        [1 0; 1i*w*CD 1];
    ZL=(AOMN(1,1)*ZOUT+AOMN(1,2))/(AOMN(2,1)*ZOUT+AOMN(2,2));


% find the coefficients to be used later for all ZL values
Den=Am(2,2)*Ap(2,2)+(Am(2,1)*Ap(2,2)+Am(2,2)*Ap(2,1))*ZL;
A11=(Am(1,2)*Ap(2,2)+(Am(1,1)*Ap(2,2)+Am(1,2)*Ap(2,1))*ZL)/Den;
A12=((Am(1,1)*Am(2,2)-Am(1,2)*Am(2,1))*ZL)/Den;
A22=(Ap(1,2)*Am(2,2)+(Ap(1,1)*Am(2,2)+Ap(1,2)*Am(2,1))*ZL)/Den;
A21=((Ap(1,1)*Ap(2,2)-Ap(1,2)*Ap(2,1))*ZL)/Den;
B11=ZL*Ap(2,2)/Den;
B12=ZL*Am(2,2)/Den;
    

IDQm=Gm*(VBm-VT);
IDQp=Gm*(VBp-VT);

il=BOmin;
ilstep=(BOmax-BOmin)/(nw-1);
% initial estimate of solution
VDSi=[(1-ConvDoh+ConvDoh*1i)*0.5;((1-ConvDoh)*1i+ConvDoh)*0.5];

for iw=1:nw 
    Vin=Ipmax/Gm*10^(il/20);
    BO(iw)=il;
    il=il+ilstep;
    Vinm=Vin*alpha;
    Vinp=Vin*beta;

    Ipm=Gm*(Vinm+VBm-VT);
    Ipp=Gm*(Vinp+VBp-VT);



    AT=[A11 A12;A21 A22];
    fun=@(VDS) myfun(Ipm,Ipp,IDQm,IDQp,VDS,No,nchko,Ne,nchke,AT,VDD,ConvDoh,y);
    [VDS,FVAL,EXITFLAG]=fsolve(fun,VDSi,options); % find the VDS value for these conditions
    if max(abs(FVAL))<1e-4
    VDSm(1,iw)=VDS(1);
    VDSp(1,iw)=VDS(2);
    VDSi=VDS;
    else
    VDSm(1,iw)=VDSi(1);
    VDSp(1,iw)=VDSi(2);
    end

        I0m(1,iw)=DrainCurrent1m(Ipm,IDQm,((1-ConvDoh)*1i+ConvDoh)*VDS(1),No,nchko);    % phase shift needed if ConvDoh=0
        I0m(1,iw)=(I0m(1,iw)+DrainCurrent1(Ipm,IDQm,((1-ConvDoh)*1i+ConvDoh)*VDS(1),Ne))/2;    % phase shift needed if ConvDoh=0

        I0p(1,iw)=DrainCurrent1m(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),No,nchko); % no phase shift needed
        I0p(1,iw)=(I0p(1,iw)+DrainCurrent1(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),Ne))/2; % no phase shift needed

        IDSm(1,iw)=(ConvDoh-(1-ConvDoh)*1i)*DrainCurrent3m(Ipm,IDQm,((1-ConvDoh)*1i+ConvDoh)*VDS(1),No,nchko); % current of main trans.
        IDSm(1,iw)=(IDSm(1,iw)+(ConvDoh-(1-ConvDoh)*1i)*DrainCurrent3(Ipm,IDQm,((1-ConvDoh)*1i+ConvDoh)*VDS(1),Ne,nchke))/2; % current of main trans. 

        IDSp(1,iw)=(-ConvDoh*1i+1-ConvDoh)*DrainCurrent3m(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),No,nchko); % current of peaking trans.
        IDSp(1,iw)=(IDSp(1,iw)+(-ConvDoh*1i+1-ConvDoh)*DrainCurrent3(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),No,nchko))/2; % current of peaking trans.

    
end

VL=B11*IDSm+B12*IDSp; % voltage at the combining node
POUT=1/2*real((VL).*conj(VL./ZL)); % output power in W
EFF=POUT./((I0m+I0p)*VDD); % efficiency

PDB=10*log10(POUT)+30; % output power in dBm

figure(2)
subplot(2,2,1)
hold off
plot(BO,PDB,'-','LineWidth',2,'Color','blue')
hold on
grid on
xlabel('Output Backoff (dB)')
ylabel('P_{out} (dBm)')
axis([BOmin,BOmax,25,45]);

subplot(2,2,2)
hold off
plot(BO,EFF,'-','LineWidth',2,'Color','blue')
hold on
grid on
ylabel('\eta')
xlabel('Output Backoff (dB)')
axis([BOmin,BOmax,0.,0.8]);

subplot(2,2,3)
hold off
plot(BO,I0m)
hold on
plot(BO,I0p)
grid on
ylabel('Main and Peak. Currents')
xlabel('Output Backoff (dB)')

axis([BOmin,BOmax,0.,0.7]);

for ii=1:NoOfSteps
    BOn(ii)=interp1(PDB,BO,PDB(nw)-BackOffSmall-StepSize*(NoOfSteps-1)+StepSize*(ii-1)); % 2dB steps
end
% save the results in BOn.mat file for use later
save('BOn.mat','BOn','NoOfSteps','zeta','kappa','nchko','nchke','C','L','Ls','Lsh','Z2','Z3','ZSh','x','x1','y','y1','y2','ySh','CA','CB','CC','CD','Z4','Z5','Z6','C1','C2','C3','C4','L1','L2','L3','VT','VDD','Gm','Q','Ropt','Roptp','ZOUT','No','Ne','VBm','VBp','alpha','beta','Ipmax','ConvDo','ConvDoh','fN','C_offset','L_offset','CA','LA','LB');
DohertyFreqResp_m2

% Nonlinear function to be solved (find the zero)
function f=sA1(Ipm,Ipp,IDQm,IDQp,VDS,No,nchko,Ne,nchke,AT,VDD,ConvDoh,y)

IDSm=(ConvDoh+(1-ConvDoh)*exp(-1i*2*y*pi))*DrainCurrent3m(Ipm,IDQm,((1-ConvDoh)*exp(1i*2*y*pi)+ConvDoh)*VDS(1),No,nchko);
IDSm=(IDSm+(ConvDoh+(1-ConvDoh)*exp(-1i*2*y*pi))*DrainCurrent3(Ipm,IDQm,((1-ConvDoh)*exp(1i*2*y*pi)+ConvDoh)*VDS(1),Ne,nchke))/2;

IDSp=(-ConvDoh*1i+1-ConvDoh)*DrainCurrent3m(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),No,nchko);
IDSp=(IDSp+(-ConvDoh*1i+1-ConvDoh)*DrainCurrent3(Ipp,IDQp,(1-ConvDoh+ConvDoh*1i)*VDS(2),Ne,nchke))/2;

    
f=VDD*VDS-AT*[IDSm;IDSp]; % vector to be zeroed
end


